ENV_KEY_SCW_CONFIG_PATH = "SCW_CONFIG_PATH"
ENV_KEY_SCW_PROFILE = "SCW_PROFILE"
ENV_KEY_SCW_ACCESS_KEY = "SCW_ACCESS_KEY"
ENV_KEY_SCW_SECRET_KEY = "SCW_SECRET_KEY"  # nosec B105
ENV_KEY_SCW_API_URL = "SCW_API_URL"
ENV_KEY_SCW_DEFAULT_ORGANIZATION_ID = "SCW_DEFAULT_ORGANIZATION_ID"
ENV_KEY_SCW_DEFAULT_PROJECT_ID = "SCW_DEFAULT_PROJECT_ID"
ENV_KEY_SCW_DEFAULT_REGION = "SCW_DEFAULT_REGION"
ENV_KEY_SCW_DEFAULT_ZONE = "SCW_DEFAULT_ZONE"

ENV_VARIABLES_TO_PROFILE = {
    ENV_KEY_SCW_ACCESS_KEY: "access_key",
    ENV_KEY_SCW_SECRET_KEY: "secret_key",
    ENV_KEY_SCW_API_URL: "api_url",
    ENV_KEY_SCW_DEFAULT_ORGANIZATION_ID: "default_organization_id",
    ENV_KEY_SCW_DEFAULT_PROJECT_ID: "default_project_id",
    ENV_KEY_SCW_DEFAULT_REGION: "default_region",
    ENV_KEY_SCW_DEFAULT_ZONE: "default_zone",
}
